// CSutilities, har överlapp med GL_utilities

#include <stdio.h>
#include <stdlib.h>
#include <sys/times.h>
#include <GL/gl.h>

static char* readFile(const char *file)
{
	FILE *fptr;
	long length;
	char *buf;

	fptr = fopen(file, "rb"); /* Open file for reading */
	if (!fptr) /* Return NULL on failure */
		return NULL;
	fseek(fptr, 0, SEEK_END); /* Seek to the end of the file */
	length = ftell(fptr); /* Find out how many bytes into the file we are */
	buf = (char*)malloc(length+1); /* Allocate a buffer for the entire length of the file and a null terminator */
	fseek(fptr, 0, SEEK_SET); /* Go back to the beginning of the file */
	fread(buf, length, 1, fptr); /* Read the contents of the file in to the buffer */
	fclose(fptr); /* Close the file */
	buf[length] = 0; /* Null terminator */
	
	return buf; /* Return the buffer */
}

// Infolog: Show result of shader compilation
static void printShaderInfoLog(GLuint obj, const char *fn)
{
	GLint infologLength = 0;
	GLint charsWritten  = 0;
	char *infoLog;

	glGetShaderiv(obj, GL_INFO_LOG_LENGTH, &infologLength);

	if (infologLength > 2)
	{
		fprintf(stderr, "[From %s:]\n", fn);
		infoLog = (char *)malloc(infologLength);
		glGetShaderInfoLog(obj, infologLength, &charsWritten, infoLog);
		fprintf(stderr, "%s\n",infoLog);
		free(infoLog);
	}
}

static void printProgramInfoLog(GLuint obj, const char *vfn)
{
	GLint infologLength = 0;
	GLint charsWritten  = 0;
	char *infoLog;

	glGetProgramiv(obj, GL_INFO_LOG_LENGTH,&infologLength);

	if (infologLength > 2)
	{
		fprintf(stderr, "[From %s:]\n", vfn);
		infoLog = (char *)malloc(infologLength);
		glGetProgramInfoLog(obj, infologLength, &charsWritten, infoLog);
		fprintf(stderr, "%s\n",infoLog);
		free(infoLog);
	}
}

// Compile a shader, return reference to it
static GLuint compileShaders(char *vs, const char *vfn)
{
	GLuint v,p;
	
	v = glCreateShader(GL_COMPUTE_SHADER);
	glShaderSource(v, 1, &vs, NULL);
	glCompileShader(v);
	
	p = glCreateProgram();//#include "opengl.h"

	glAttachShader(p,v);
	glLinkProgram(p);
	glUseProgram(p);
	
	printShaderInfoLog(v, vfn);
	printProgramInfoLog(p, vfn);
	
	return p;
}

GLuint loadComputeShader(const char * filename)
{
    char *vs;
       
	vs  = readFile(filename);

	return compileShaders(vs,filename);
}

